@@ -13,7 +13,7 @@ from PyQt5.Qt import (
     QCheckBox, QComboBox, QDialog, QDialogButtonBox, QDoubleSpinBox, QFormLayout,
     QFrame, QHBoxLayout, QIcon, QLabel, QLineEdit, QListWidget, QPlainTextEdit,
     QPushButton, QScrollArea, QSize, QSizePolicy, QSpinBox, Qt, QTabWidget, QTimer,
-    QUrl, QVBoxLayout, QWidget, pyqtSignal
+    QToolButton, QUrl, QVBoxLayout, QWidget, pyqtSignal
 )
 
 from calibre import as_unicode
@@ -103,6 +103,7 @@ class Text(QLineEdit):
 
     def __init__(self, name, layout):
         QLineEdit.__init__(self)
+        self.setClearButtonEnabled(True)
         opt = options[name]
         self.textChanged.connect(self.changed_signal.emit)
         init_opt(self, opt, layout)
@@ -114,6 +115,40 @@ class Text(QLineEdit):
         self.setText(type(u'')(val or ''))
 
 
+class Path(QWidget):
+
+    changed_signal = pyqtSignal()
+
+    def __init__(self, name, layout):
+        QWidget.__init__(self)
+        self.dname = name
+        opt = options[name]
+        self.l = l = QHBoxLayout(self)
+        l.setContentsMargins(0, 0, 0, 0)
+        self.text = t = QLineEdit(self)
+        t.setClearButtonEnabled(True)
+        t.textChanged.connect(self.changed_signal.emit)
+        l.addWidget(t)
+
+        self.b = b = QToolButton(self)
+        l.addWidget(b)
+        b.setIcon(QIcon(I('document_open.png')))
+        b.setToolTip(_("Browse for the file"))
+        b.clicked.connect(self.choose)
+        init_opt(self, opt, layout)
+
+    def get(self):
+        return self.text.text().strip() or None
+
+    def set(self, val):
+        self.text.setText(type(u'')(val or ''))
+
+    def choose(self):
+        ans = choose_files(self, 'choose_path_srv_opts_' + self.dname, _('Choose a file'), select_only_single_file=True)
+        if ans:
+            self.set(ans[0])
+
+
 class Choices(QComboBox):
 
     changed_signal = pyqtSignal()
@@ -146,6 +181,7 @@ class AdvancedTab(QWidget):
         self.l = l = QFormLayout(self)
         l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
         self.widgets = []
+        self.widget_map = {}
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         for name in sorted(options, key=lambda n: options[n].shortdoc.lower()):
             if name in ('auth', 'port', 'allow_socket_preallocation', 'userdb'):
@@ -161,9 +197,12 @@ class AdvancedTab(QWidget):
                 w = Float
             else:
                 w = Text
+                if name in ('ssl_certfile', 'ssl_keyfile'):
+                    w = Path
             w = w(name, l)
             setattr(self, 'opt_' + name, w)
             self.widgets.append(w)
+            self.widget_map[name] = w
 
     def genesis(self):
         opts = server_config()
@@ -175,10 +214,16 @@ class AdvancedTab(QWidget):
         for w in self.widgets:
             w.set(w.default_val)
 
+    def get(self, name):
+        return self.widget_map[name].get()
+
     @property
     def settings(self):
         return {w.name: w.get() for w in self.widgets}
 
+    @property
+    def has_ssl(self):
+        return bool(self.get('ssl_certfile')) and bool(self.get('ssl_keyfile'))
 
 # }}}
 
@@ -240,8 +285,20 @@ class MainTab(QWidget):  # {{{
             if name == 'show_logs':
                 h.addStretch(10)
             h.addWidget(b)
+        self.ip_info = QLabel(self)
+        self.update_ip_info()
+        from calibre.gui2.ui import get_gui
+        get_gui().iactions['Connect Share'].share_conn_menu.server_state_changed_signal.connect(self.update_ip_info)
+        l.addSpacing(10)
+        l.addWidget(self.ip_info)
         l.addStretch(10)
 
+    def update_ip_info(self):
+        from calibre.gui2.ui import get_gui
+        t = get_gui().iactions['Connect Share'].share_conn_menu.ip_text
+        t = t.strip().strip('[]')
+        self.ip_info.setText(_('Content server listening at: %s') % t)
+
     def genesis(self):
         opts = server_config()
         self.opt_auth.setChecked(opts.auth)
@@ -252,7 +309,7 @@ class MainTab(QWidget):  # {{{
 
     def change_auth_desc(self):
         self.auth_desc.setText(
-            _('Remember to create some user accounts in the "Users" tab')
+            _('Remember to create some user accounts in the "User accounts" tab')
             if self.opt_auth.isChecked() else _(
                 'Requiring a username/password prevents unauthorized people from'
                 ' accessing your calibre library. It is also needed for some features'
@@ -273,6 +330,8 @@ class MainTab(QWidget):  # {{{
         from calibre.gui2.ui import get_gui
         gui = get_gui()
         is_running = gui.content_server is not None and gui.content_server.is_running
+        self.ip_info.setVisible(is_running)
+        self.update_ip_info()
         self.start_server_button.setEnabled(not is_running)
         self.stop_server_button.setEnabled(is_running)
         self.test_server_button.setEnabled(is_running)
@@ -960,10 +1019,14 @@ class ConfigWidget(ConfigWidgetBase):
         self.stopping_msg.accept()
 
     def test_server(self):
-        prefix = self.advanced_tab.opt_url_prefix.text().strip()
-        open_url(
-            QUrl('http://127.0.0.1:' + str(self.main_tab.opt_port.value()) + prefix)
-        )
+        prefix = self.advanced_tab.get('url_prefix') or ''
+        protocol = 'https' if self.advanced_tab.has_ssl else 'http'
+        lo = self.advanced_tab.get('listen_on') or '0.0.0.0'
+        lo = {'0.0.0.0': '127.0.0.1', '::':'::1'}.get(lo)
+        url = '{protocol}://{interface}:{port}{prefix}'.format(
+            protocol=protocol, interface=lo,
+            port=self.main_tab.opt_port.value(), prefix=prefix)
+        open_url(QUrl(url))
 
     def view_server_logs(self):
         from calibre.srv.embedded import log_paths
