@@ -858,8 +858,8 @@ class WordsView(QTableView):
             for s in dictionaries.suggestions(*w):
                 cm.addAction(s, partial(self.change_to.emit, w, s))
 
-        m.addAction(_('Ignore/Unignore all selected words'), self.ignore_all)
-        a = m.addAction(_('Add/Remove all selected words'))
+        m.addAction(_('Ignore/un-ignore all selected words'), self.ignore_all)
+        a = m.addAction(_('Add/remove all selected words'))
         am = QMenu()
         a.setMenu(am)
         for dic in sorted(dictionaries.active_user_dictionaries, key=lambda x:sort_key(x.name)):
@@ -1104,6 +1104,11 @@ class SpellCheck(Dialog):
                     self.suggested_list.setCurrentItem(item)
                     self.suggested_word.setText(s)
                     word_suggested = True
+                if s is current_word:
+                    f = item.font()
+                    f.setItalic(True)
+                    item.setFont(f)
+                    item.setToolTip(_('The original word'))
             if not word_suggested:
                 self.suggested_word.setText(current_word)
 
